/*
Pieni demosovellus on tehty app-komponenttiin, mikä
on yleensä huono käytänne, sillä app-komponentin tulisi
toimia vain muiden komponenttien säiliönä. Nyt voidaan
kuitenkin tehdä näin, koska kyseessä on vain pieni demo.

Komponentin luokka on Controller tai ViewModel. Se välittää
datan Viewiin eli HTML-templaattin. Se myös tuottaa Viewin
Modelin eli tuottaa muuttujien arvot, jotka on bindattu
Viewiin.
*/

import { Component, OnInit } from '@angular/core';
import { HttpService } from './http.service';
import { Product } from './product';
import { FormsModule } from '@angular/forms';

@Component({
  selector: 'app-root',
  standalone: true,
  imports: [FormsModule],
  templateUrl: './app.component.html',
  styleUrl: './app.component.css',
})
export class AppComponent implements OnInit{
  productTitle: string | undefined;
  productPrice: string | undefined;
  products: Product[] | undefined;

  // injektoidaan service konstruktorin kautta
  constructor(private httpservice: HttpService) {}
  
  ngOnInit(): void {
    // Kaikki tuotteet haetaan products-muuttujaan aina kun sovellus käynnistyy
    this.getProducts();
  }

  /* Komponentin metodi on saman niminen kuin servicen metodi.
     Metodia kutsutaan ngOnInit-hookissa */
  getProducts() {
    this.httpservice.getProducts().subscribe((data) => {
      this.products = data;
    });
  }

  /* Komponentin metodi on saman niminen kuin servicen metodi.
     Sitä kutsutaan HTML-templaatissa */
  getProductByID(formValue: { productID: string }) {
    /* haetaan tuote servicen getProductByID -metodin avulla
       Se palauttaa observablen, joka tilataan subscribe-metodilla,jonka
       argumenttina on anonyymi funktio josta saadaan data.*/
    this.httpservice.getProductByID(formValue.productID).subscribe((data) => {
      this.productTitle = data.title;
      this.productPrice = `$` + data.price;
    });
  }
  /* Sama homma fetchillä. Tämä ei toimi kuin oikean palvelimen 
  kanssa, sillä in-memory-web-apissa ei ole fetch()-tukea. 
  Mutta toimii ang-httpservice-server -koeserverillä portista localhost:3000 */
  getProductByIDFetch(formValue: { productID: string }) {
    /* haetaan tuote servicen getProductByIDFetch -metodin avulla
       metodi palauttaa promisen,joka suoritetaan then-metodilla,jonka
       argumenttina on anonyymi funktio josta saadaan data. */
    this.httpservice
      .getProductByIDFetch(formValue.productID)
      .then((response) => response.json())
      .then((data) => {
        this.productTitle = data.title;
        this.productPrice = `$` + data.price;
      });
  }
}
