/*
Servicen käyttö datan hakemiseen sovellukseen api-rajapintojen kautta
on hyvä käytänne. Datan hakemista suoraan komponenttiin ei suositella.

Angularissa ulkopuoliseen datalähteeseen otetaan yhteys oletuksena 
HttpClient-luokasta luotavalla oliolla. Kyseessä on XHR-olio.
Angularissa voidaan yhtä hyvin käyttää myös fetch()-metodia.
Molemmat tavat hakea tietoa on esitetty alla.
*/
import { HttpClient } from '@angular/common/http'; // http-olio hakee datan
import { Injectable } from '@angular/core';
import { Observable } from 'rxjs'; // data haetaan asynkronisesti observablena
import { catchError } from 'rxjs/operators'; // virheenkäsittelyä varten
import { Product } from './product';

@Injectable({
  providedIn: 'root'
})
export class HttpService {

  apiurl = 'api/products'; // valepalvelimen eli in-memory-web-apin osoite
  // apiurl = 'http://localhost:3000/products'; // oikean palvelimen osoite

  // HttpClient-luokasta syntyy http-olio konstruktorissa
  // Tämä on dependency injection
  constructor(private http: HttpClient) { }
  
  // Tehdään palvelimelle pyyntö jolla haetaan products-taulukko observablena
  getProducts(): Observable<Product[]> {
    return this.http.get<Product[]>(this.apiurl);
    //virheenkäsittely voitaisiin tehdä tähän
  }

  /* http-olion get-metodi palauttaa observablen.
  Palautettavaan observableen on yhdistetty
  pipellä virheenkäsittely. Yksinkertaistamisen vuoksi 
  on käytetty Observablessa any-tietotyyppiä.
  */
  getProductByID(productid: string): Observable<any> {
    return this.http.get(this.apiurl + '/' + productid).pipe(
      catchError((error) => {
        console.error('Tapahtui virhe: ', error);
        return (error.message || error);
      }));
  }

  /* Sama homma kuin yllä toteutettuna fetchillä.
  Fetch palauttaa promisen, joka välitetään komponentille.
  Yksinkertaistamisen vuoksi on käytetty Promisessa any-tietotyyppiä.
  Mukana on samantyyppinen virheenkäsittely. Huomaa, että fetch()
  ei toimi in-memory-web-apin kanssa, mutta toimii kun käytetään
  oikeaa serveriä http://localhost:3000/products -osoitteesta.
  */
  getProductByIDFetch(productid: string): Promise<any> {
    return fetch(this.apiurl + '/' + productid)
      .catch((error) => {
        console.error('Tapahtui virhe: ', error);
        return (error.message || error);
      });
  }
}
